import typer

from claix import __version__


# Version callback
def version_callback(show_version: bool):
    if show_version:
        typer.echo(__version__)
        raise typer.Exit()


# OpenAI API key callback
def openai_api_key_callback(api_key: str):
    if api_key is None:
        typer.echo(typer.style("Error:", fg=typer.colors.RED, bold=True))
        typer.echo("OPENAI_API_KEY environment variable is necessary to use Claix.")
        raise typer.Exit()
    return api_key


# Instructions callback
def instructions_callback(instructions: list[str]) -> str:
    if not instructions:
        typer.echo(typer.style("Error:", fg=typer.colors.RED, bold=True))
        typer.echo(
            "No instructions provided. Claix needs a set of instructions to generate commands.\n"
        )
        typer.echo(typer.style("Example Usage:", fg=typer.colors.BLUE, bold=True))
        typer.echo(
            "claix list all docker containers\nclaix show me active network interfaces"
        )
        raise typer.Exit()
    return instructions



import time
from typing import Optional
import instructor
from pydantic import BaseModel
from openai import OpenAI


class ClaixCommand(BaseModel):
    """
    A model describing important attributes of a CLI command.

    Attributes:
        is_command (bool): True if the response is a command, False otherwise 
            (e.g., if the output is a dot or is empty).
        command (Optional[str]): The string command to execute.
        explanation (Optional[str]): The explanation of the command.
    """
    is_command: bool
    command: Optional[str]
    explanation: Optional[str]


class Claix:
    def __init__(self, assistant_id: str, thread_id: str = None):
        self.assistant_id = assistant_id
        self.thread_id = thread_id
        self.client = OpenAI()

    def __call__(self, prompt, thread_id=None):
        if not thread_id:
            thread_id = self.thread_id

        self.create_thread_message(prompt, thread_id)

        run = self.run_thread(thread_id)

        run = self.wait_for_run(run)

        text_response: str = self.get_last_message(thread_id)

        claix_command: ClaixCommand = self.populate_model_from_text(ClaixCommand, text_response)

        if not claix_command.command:
            claix_command.is_command = False

        return claix_command

    def get_fixed_command(
        self, instructions: str, proposed_command: str, command_run_result
    ) -> ClaixCommand:
        error_prompt = (
            f"I want to: '{instructions}'\n"
            f"I tried '{proposed_command}'\n"
            f"but got this error: '{command_run_result.stderr}'\n\n"
            f"Having this error in mind, fix my original command of '{proposed_command}' "
            f"or give me a new command to solve: '{instructions}'"
        )

        command: ClaixCommand = self(error_prompt)

        return command

    def get_revised_command(
        self,
        revision_instructions: str,
        original_instructions: str,
        proposed_command: str,
    ) -> str:
        revision_prompt = (
            f"I want to: {original_instructions}\n"
            f"The proposed solution was: {proposed_command}\n\n"
            f"but I want to modify this solution considering this: {revision_instructions}\n\n"
            f"please give me a new solution considering that"
        )

        revised_solution: str = self(revision_prompt)

        return revised_solution

    @staticmethod
    def create_assistant(
        name: str,
        instructions: str,
        model: str = "gpt-4-1106-preview",
        tools: list[dict] = [{"type": "code_interpreter"}, {"type": "retrieval"}],
    ):
        client = OpenAI()
        assistant = client.beta.assistants.create(
            name=name,
            instructions=instructions,
            tools=tools,
            model=model,
        )
        return assistant

    @staticmethod
    def create_thread():
        client = OpenAI()
        thread = client.beta.threads.create()
        return thread

    @staticmethod
    def populate_model_from_text(model: BaseModel, text: str) -> BaseModel:
        client = instructor.patch(OpenAI())
        """Populates a pydantic model from a text."""
        response: model = client.chat.completions.create(
            model="gpt-3.5-turbo",
            response_model=model,
            messages=[
                {"role": "user", "content": text},
            ],
            max_retries=3,
        )
        return response

    def create_thread_message(self, prompt: str, thread_id: str):
        message = self.client.beta.threads.messages.create(
            thread_id=thread_id,
            role="user",
            content=prompt,
        )
        return message

    def run_thread(self, thread_id: str):
        run = self.client.beta.threads.runs.create(
            thread_id=thread_id,
            assistant_id=self.assistant_id,
        )
        return run

    def update_run(self, run):
        run = self.client.beta.threads.runs.retrieve(
            run_id=run.id,
            thread_id=run.thread_id,
        )
        return run

    def wait_for_run(self, run):
        while run.status != "completed":
            run = self.update_run(run)
            time.sleep(0.1)
        return run

    def get_thread_messages(self, thread_id: str):
        messages = self.client.beta.threads.messages.list(
            thread_id=thread_id,
        )
        return messages

    def get_last_message(self, thread_id: str):
        messages = self.get_thread_messages(thread_id)
        return list(messages)[0].content[0].text.value

    def add_files_to_assistant(self, file_ids: str | list[str]):
        if isinstance(file_ids, str):
            file_ids = [file_ids]

        self.client.beta.assistants.update(
            self.assistant_id,
            file_ids=file_ids,
        )



from enum import Enum, auto
import typer
from claix.ai import Claix, ClaixCommand
from claix.typer_utils import (
    version_callback,
    instructions_callback,
    openai_api_key_callback,
)
from claix.utils import (
    get_or_create_default_assistant_id,
    get_or_create_default_thread_id,
    ask_user_if_run_revise_or_exit,
    run_shell_command,
    simulate_clear,
    Action,
)
from rich.console import Console

app = typer.Typer(name="claix", no_args_is_help=True, add_completion=False)


class State(Enum):
    PROCESS_INSTRUCTIONS = auto()
    GATHER_REVISION = auto()
    PRINT_EXPLANATION = auto()
    USER_DECISION = auto()
    RUN_COMMAND = auto()
    HANDLE_FAILURE = auto()
    EXIT = auto()


@app.command(help="Turns your instructions into CLI commands ready to execute.")
def main(
    initial_instructions: list[str] = typer.Argument(
        None,
        callback=instructions_callback,
        help="The instructions that you want to execute.",
    ),
    show_version: bool = typer.Option(
        None,
        "--version",
        "-v",
        callback=version_callback,
        is_eager=True,
        help="Show the version and exit.",
    ),
    openai_api_key: str = typer.Option(
        None,
        hidden=True,
        callback=openai_api_key_callback,
        envvar="OPENAI_API_KEY",
        help="The OpenAI API key.",
    ),
):
    instructions = " ".join(initial_instructions)
    error_iterations = 0

    assistant_id = get_or_create_default_assistant_id()
    thread_id = get_or_create_default_thread_id()

    console = Console()
    claix = Claix(assistant_id, thread_id)
    state = State.PROCESS_INSTRUCTIONS

    start = True
    while state != State.EXIT:
        if state == State.PROCESS_INSTRUCTIONS:
            with console.status("Thinking...", spinner="dots"):
                claix_command: ClaixCommand = claix(instructions)

            if not claix_command.is_command:
                typer.echo(
                    typer.style(
                        "Claix Response: I don't know how to solve this problem, exiting",
                        fg=typer.colors.RED,
                        bold=True,
                    )
                )
                state = State.EXIT
            else:
                typer.echo(
                    typer.style("Proposed Command:", fg=typer.colors.GREEN, bold=True)
                )
                typer.echo(f"{claix_command.command}\n")
                state = State.USER_DECISION

        elif state == State.USER_DECISION:
            user_action = ask_user_if_run_revise_or_exit()
            #simulate_clear(console)
            if user_action == Action.EXIT:
                state = State.EXIT
            elif user_action == Action.REVISE:
                state = State.GATHER_REVISION
            elif user_action == Action.EXPLAIN:
                state = State.PRINT_EXPLANATION
            elif user_action == Action.RUN:
                state = State.RUN_COMMAND

        elif state == State.GATHER_REVISION:
            instructions = typer.prompt("Enter your revision")
            state = State.PROCESS_INSTRUCTIONS

        elif state == State.PRINT_EXPLANATION:
            typer.echo(
                typer.style("Explanation:", fg=typer.colors.BLUE, bold=True)
            )
            typer.echo(f"{claix_command.explanation}\n")
            state = State.USER_DECISION

        elif state == State.RUN_COMMAND:
            with console.status("Running command...", spinner="dots"):
                command_run_result = run_shell_command(claix_command.command)
            if command_run_result.returncode == 0:
                if command_run_result.stdout:
                    typer.echo(typer.style("Output:", fg=typer.colors.BLUE, bold=True))
                    typer.echo(command_run_result.stdout)
                else:
                    typer.echo(
                        typer.style("No output.", fg=typer.colors.BLUE, bold=True)
                    )
                state = State.EXIT
            else:
                typer.echo(
                    typer.style("Command failed...", fg=typer.colors.RED, bold=True)
                )
                state = State.HANDLE_FAILURE
                error_iterations = 0

        elif state == State.HANDLE_FAILURE:
            if error_iterations > 2:
                typer.echo(
                    typer.style(
                        "Too many errors, exiting", fg=typer.colors.RED, bold=True
                    )
                )
                state = State.EXIT
                break
            with console.status("Thinking", spinner="dots"):
                claix_command: ClaixCommand = claix.get_fixed_command(
                    instructions, claix_command.command, command_run_result
                )
            if not claix_command.is_command:
                typer.echo(
                    typer.style(
                        "Claix Response: I don't know how to solve this problem, exiting",
                        fg=typer.colors.RED,
                        bold=True,
                    )
                )
                state = State.EXIT
            else:
                typer.echo(
                    typer.style("Proposed Solution:", fg=typer.colors.GREEN, bold=True)
                )
                typer.echo(f"{claix_command.command}\n")

                user_action = ask_user_if_run_revise_or_exit()
                #simulate_clear(console)
                if user_action == Action.EXIT:
                    state = State.EXIT
                elif user_action == Action.REVISE:
                    state = State.GATHER_REVISION
                elif user_action == Action.EXPLAIN:
                    state = State.PRINT_EXPLANATION
                elif user_action == Action.RUN:
                    state = State.RUN_COMMAND
                    error_iterations += 1


if __name__ == "__main__":
    app()



import os
import shelve
import subprocess
from pathlib import Path
from enum import Enum, auto
import rich
import inquirer
from claix.ai import Claix


USER_OS = "Windows" if os.name == "nt" else "Posix"

DEFAULT_INSTRUCTIONS = f"""
Claix exclusively provides CLI command translations in plain text for any OS but specially {USER_OS}, with no code blocks or additional formatting. When a user's input aligns with {USER_OS} CLI commands, Claix responds with the exact command in simple text followed by a brief explanation of its function in simple text. If the input is unrelated to {USER_OS} CLI commands, Claix replies with a single '.' to maintain focus on its primary role.

This GPT avoids any execution or simulation of CLI commands and does not engage in discussions beyond {USER_OS} CLI command translation and explanation. Claix's responses are concise, delivering {USER_OS} CLI commands along with succinct explanations in an unembellished, clear format, ensuring users receive direct and unformatted command syntax and understanding for their {USER_OS}-related inquiries.
"""

db_path = Path.home() / ".claix" / "db"
db_path.parent.mkdir(exist_ok=True)


class Action(Enum):
    RUN = auto()
    REVISE = auto()
    EXPLAIN = auto()
    EXIT = auto()


    def __str__(self):
        # Return the string representation for the prompt
        emojis = {
            Action.RUN: "\U00002705 Run Command",
            Action.REVISE: "\U0001F4DD Revise Command",
            Action.EXPLAIN: "\U0001F4D6 Explain Command",
            Action.EXIT: "\U0000274C Exit",
        }
        return emojis[self]


def ask_user_if_run_revise_or_exit() -> Action:
    questions = [
        inquirer.List(
            "action",
            message="Choose an action",
            choices=list(Action),
        ),
    ]
    action_response = inquirer.prompt(questions, raise_keyboard_interrupt=True)[
        "action"
    ]

    return Action(action_response)


def run_shell_command(command):
    command_run_result = subprocess.run(
        command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, shell=True
    )
    return command_run_result


def get_assistant_id(assistant: str = "default"):
    with shelve.open(str(db_path)) as db:
        try:
            return db["assistants"][assistant]["id"]
        except KeyError:
            return None


def set_assistant_id(assistant_id, assistant="default"):
    with shelve.open(str(db_path)) as db:
        assistants = db.get("assistants", {})
        default = assistants.get(assistant)
        if default:
            db["assistants"][assistant]["id"] = assistant_id
        else:
            db["assistants"] = {assistant: {"id": assistant_id}}
    return assistant_id


def get_or_create_default_assistant_id():
    assistant_id = get_assistant_id()
    if not assistant_id:
        assistant = Claix.create_assistant(
            name="default",
            instructions=DEFAULT_INSTRUCTIONS,
        )
        assistant_id = set_assistant_id(assistant.id, assistant="default")
    return assistant_id


def get_thread_id(thread: str = "default"):
    with shelve.open(str(db_path)) as db:
        try:
            return db["threads"][thread]["id"]
        except KeyError:
            return None


def set_thread_id(thread_id, thread="default"):
    with shelve.open(str(db_path)) as db:
        threads = db.get("threads", {})
        default = threads.get(thread)
        if default:
            db["threads"][thread]["id"] = thread_id
        else:
            db["threads"] = {thread: {"id": thread_id}}
    return thread_id


def get_or_create_default_thread_id():
    thread_id = get_thread_id()
    if not thread_id:
        thread = Claix.create_thread()
        thread_id = set_thread_id(thread.id, thread="default")
    return thread_id


def simulate_clear(console: rich.console.Console):
    """
    Simulates clearing the console by printing enough new lines to push old content out of view,
    then moves the cursor back to the top of the console window.
    """
    height = console.size.height
    print("\n" * height, end="")  # Print newlines to push content out of view
    print(
        f"\033[{height}A", end=""
    )  # Move the cursor back up to the top of the console window
